@@ -186,22 +186,23 @@ fn similarity_filter_is_accurate() {
     resp.set_text("sitting");
 
     let mut filter = SimilarityFilter {
-        text: FuzzyHash::new("kitten").to_string(),
+        hash: FuzzyHash::new("kitten").to_string(),
         threshold: 95,
+        original_url: "".to_string(),
     };
 
     // kitten/sitting is 57% similar, so a threshold of 95 should not be filtered
     assert!(!filter.should_filter_response(&resp));
 
     resp.set_text("");
-    filter.text = String::new();
+    filter.hash = String::new();
     filter.threshold = 100;
 
     // two empty strings are the same, however ssdeep doesn't accept empty strings, expect false
     assert!(!filter.should_filter_response(&resp));
 
     resp.set_text("some data to hash for the purposes of running a test");
-    filter.text = FuzzyHash::new("some data to hash for the purposes of running a te").to_string();
+    filter.hash = FuzzyHash::new("some data to hash for the purposes of running a te").to_string();
     filter.threshold = 17;
 
     assert!(filter.should_filter_response(&resp));
@@ -211,20 +212,58 @@ fn similarity_filter_is_accurate() {
 /// just a simple test to increase code coverage by hitting as_any and the inner value
 fn similarity_filter_as_any() {
     let filter = SimilarityFilter {
-        text: String::from("stuff"),
+        hash: String::from("stuff"),
         threshold: 95,
+        original_url: "".to_string(),
     };
 
     let filter2 = SimilarityFilter {
-        text: String::from("stuff"),
+        hash: String::from("stuff"),
         threshold: 95,
+        original_url: "".to_string(),
     };
 
     assert!(filter.box_eq(filter2.as_any()));
 
-    assert_eq!(filter.text, "stuff");
+    assert_eq!(filter.hash, "stuff");
     assert_eq!(
         *filter.as_any().downcast_ref::<SimilarityFilter>().unwrap(),
         filter
     );
 }
+
+#[test]
+/// test correctness of FeroxFilters::remove
+fn remove_function_works_as_expected() {
+    let data = FeroxFilters::default();
+    assert!(data.filters.read().unwrap().is_empty());
+
+    (0..8).for_each(|i| {
+        data.push(Box::new(WordsFilter { word_count: i })).unwrap();
+    });
+
+    // remove removes index-1 from the vec, zero is skipped, and out-of-bounds indices are skipped
+    data.remove(&mut [0]);
+    assert_eq!(data.filters.read().unwrap().len(), 8);
+
+    data.remove(&mut [10000]);
+    assert_eq!(data.filters.read().unwrap().len(), 8);
+
+    // removing 0, 2, 4
+    data.remove(&mut [1, 3, 5]);
+
+    assert_eq!(data.filters.read().unwrap().len(), 5);
+
+    let expected = vec![
+        WordsFilter { word_count: 1 },
+        WordsFilter { word_count: 3 },
+        WordsFilter { word_count: 5 },
+        WordsFilter { word_count: 6 },
+        WordsFilter { word_count: 7 },
+    ];
+
+    for filter in data.filters.read().unwrap().iter() {
+        let downcast = filter.as_any().downcast_ref::<WordsFilter>().unwrap();
+        assert!(expected.contains(downcast));
+    }
+}
